<?php

namespace common\logic\Document\DeliveryNote\Service;

use common\logic\Distribution\Distribution\Model\Distribution;
use common\logic\Document\DeliveryNote\Model\DeliveryNote;
use common\logic\Document\Document\Service\DocumentBuilder;
use common\logic\PointSale\PointSale\Model\PointSale;
use common\logic\User\User\Repository\UserRepository;
use common\logic\User\User\Service\UserSolver;
use yii\base\ErrorException;

class DeliveryNoteBuilder extends DocumentBuilder
{
    protected UserSolver $userSolver;
    protected UserRepository $userRepository;

    public function loadDependencies(): void
    {
        parent::loadDependencies();
        $this->userSolver = $this->loadService(UserSolver::class);
        $this->userRepository = $this->loadService(UserRepository::class);
    }

    public function instanciateDeliveryNote(): DeliveryNote
    {
        $deliveryNote = new DeliveryNote();

        return $deliveryNote;
    }

    public function createDeliveryNoteForPointSale(PointSale $pointSale, Distribution $distribution): DeliveryNote
    {
        $deliveryNote = $this->instanciateDeliveryNote();
        $this->initDocumentProducer($deliveryNote);
        $this->initTaxCalculationMethod($deliveryNote);
        $this->initDeliveryNoteName($deliveryNote, $pointSale, $distribution);
        $this->initDeliveryNoteUserFromPointSale($deliveryNote, $pointSale);
        $this->initDeliveryNoteAddress($deliveryNote);

        $deliveryNote->save();

        return $deliveryNote;
    }

    public function initDeliveryNoteUserFromPointSale(DeliveryNote $deliveryNote, PointSale $pointSale)
    {
        $deliveryNote->id_user = $pointSale->id_user;
    }

    public function initDeliveryNoteName(DeliveryNote $deliveryNote, PointSale $pointSale, Distribution $distribution)
    {
        $deliveryNote->name = 'Bon de livraison ' . $pointSale->name .
            ' (' . date('d/m/Y', strtotime($distribution->date)) . ')';
    }

    public function initDeliveryNoteAddress(DeliveryNote $deliveryNote)
    {
        $user = $this->userRepository->findOneUserById($deliveryNote->id_user);
        if($user) {
            $deliveryNote->address = $this->userSolver->getFullAddress($user);
        }
        else {
            throw new ErrorException("L'utilisateur du bon de livraison n'est pas défini.");
        }
    }
}